@@ -1,12 +1,12 @@
 package Feersum;
-use 5.008007;
+use 5.014;
 use strict;
 use warnings;
 use EV ();
 use Carp ();
 use Socket ();
 
-our $VERSION = '1.506_12';
+our $VERSION = '1.506_13';
 
 require Feersum::Connection;
 require Feersum::Connection::Handle;
@@ -86,10 +86,6 @@ sub use_socket {
 # callbacks.
 sub DIED { Carp::confess "DIED: $@"; }
 
-# TLS convenience: set_tls on the just-added listener
-# Usage: $server->use_socket($sock); $server->set_tls(cert_file => $path, key_file => $path);
-# The XS method set_tls is already available directly.
-
 1;
 __END__
 
@@ -158,11 +154,6 @@ buffer.  For response data, references to scalars are kept in order to avoid
 copying the string values (once the data is written to the socket, the
 reference is dropped and the data is garbage collected).
 
-A trivial hello-world handler can process in excess of 5000 requests per
-second on a 4-core Intel(R) Xeon(R) E5335 @ 2.00GHz using TCPv4 on the
-loopback interface, OS Ubuntu 6.06LTS, Perl 5.8.7.  Your mileage will likely
-vary.
-
 For even faster results, Feersum can support very simple pre-forking (See
 L<feersum>, L<Feersum::Runner> or L<Plack::Handler::Feersum> for details).
 
@@ -179,12 +170,10 @@ Feersum will use "Transfer-Encoding: chunked" for HTTP/1.1 clients and
 streaming isn't part of the HTTP/1.0 or 1.1 spec, but many browsers and agents
 support it anyway.
 
-Currently POST/PUT does not stream input, but read() can be called on
-C<psgi.input> to get the body (which has been buffered up before the request
-callback is called and therefore will never block).  Likely C<read()> will
-change to raise EAGAIN responses and allow for a callback to be registered on
-the arrival of more data. (The C<psgix.input.buffered> env var is set to
-reflect this).
+POST/PUT request bodies (including chunked transfer-encoding) are fully
+buffered before the request callback fires, so C<read()> on C<psgi.input>
+will never block.  (The C<psgix.input.buffered> env var is set to reflect
+this).
 
 =head2 PSGI interface
 
@@ -202,9 +191,10 @@ The env hash passed in will always have the following keys in addition to
 dynamic ones:
 
     psgi.version      => [1,1],
-    psgi.nonblocking  => 1,
-    psgi.multithread  => '', # i.e. false
-    psgi.multiprocess => '',
+    psgi.nonblocking  => 1,        # PL_sv_yes
+    psgi.multithread  => !1,       # PL_sv_no (false)
+    psgi.multiprocess => !1,       # PL_sv_no (false)
+    psgi.run_once     => !1,       # PL_sv_no (false)
     psgi.streaming    => 1,
     psgi.errors       => \*STDERR,
     SCRIPT_NAME       => "",
@@ -279,9 +269,10 @@ buffered in some way.
 
 Feersum currently buffers the entire input in memory calling the callback.
 
-B<Feersum's input behaviour MAY eventually change to not be
-psgix.input.buffered!>  Likely, a C<poll_cb()> method similar to how the
-writer handle works could be registered to have input "pushed" to the app.
+Feersum also supports a C<poll_cb()> method on the reader handle for
+incremental (streaming) input.  When C<poll_cb> is active, Feersum delivers
+body data to the callback as it arrives.  C<psgix.input.buffered> remains
+C<1> because data is still buffered in memory before delivery.
 
 =item psgix.output.guard
 
@@ -302,20 +293,53 @@ L<Web::Hippie> and websockets.  C<psgix.io> is defined as part of PSGI 1.1.
 To obtain the L<IO::Socket> corresponding to this connection, read this
 environment variable.
 
-The underlying file descriptor will have C<O_NONBLOCK>, C<TCP_NODELAY>,
-C<SO_OOBINLINE> enabled and C<SO_LINGER> disabled.
+For plain (non-TLS) connections the returned L<IO::Socket::INET> wraps the raw
+TCP file descriptor, which will have C<O_NONBLOCK>, C<TCP_NODELAY>,
+C<SO_OOBINLINE> enabled and C<SO_LINGER> disabled.  For TLS and HTTP/2
+connections, C<psgix.io> returns a Unix socketpair that relays data through the
+TLS/H2 layer transparently.
 
 PSGI apps B<MUST> use a C<psgi.streaming> response so that Feersum doesn't try
-to flush and close the connection.  Additionally, the "respond" parameter to
-the streaming callback B<MUST NOT> be called for the same reason.
+to flush and close the connection.  For HTTP/1 connections, the "respond"
+parameter to the streaming callback B<MUST NOT> be called for the same reason.
+For HTTP/2 Extended CONNECT, calling the responder with a C<200> response is
+the correct way to accept the tunnel.
 
     my $env = shift;
     return sub {
         my $fh = $env->{'psgix.io'};
-        syswrite $fh, "HTTP/1.1 200 OK\r\n\r\nHello from raw socket";
-        close $fh;
+        syswrite $fh, "HTTP/1.1 101 Switching Protocols\r\n"
+                     . "Upgrade: myproto\r\nConnection: Upgrade\r\n\r\n";
+        # ... bidirectional I/O on $fh ...
     };
 
+B<HTTP/2 note:> For H2 Extended CONNECT tunnels, Feersum automatically sends
+200 HEADERS to accept the tunnel and silently swallows the HTTP/1.1 101
+response written by the app.  This means the same handler code works for both
+H1 and H2 without branching.  See L</HTTP/2 support> for details.
+
+=item psgix.h2.trailers
+
+An array-ref of C<[name, value]> pairs containing HTTP/2 trailer headers
+received with the request.  Only present for HTTP/2 requests that included
+trailers.  Absent for HTTP/1.x requests and H2 requests without trailers.
+
+=item psgix.h2.extended_connect
+
+Set to C<1> on HTTP/2 Extended CONNECT streams (RFC 8441).  Absent for
+all other request types including plain HTTP/2 requests.
+
+=item psgix.h2.protocol
+
+Present on HTTP/2 Extended CONNECT streams.  Contains the value of the
+H2 C<:protocol> pseudo-header (e.g. C<"websocket">).
+
+=item psgix.proxy_tlvs
+
+Present when the connection arrived via PROXY protocol v2 with TLV
+extensions.  A hash ref mapping TLV type numbers to their raw values.
+See L<Feersum::Connection/proxy_tlvs> for details.
+
 =back
 
 =head2 The Feersum-native interface
@@ -435,7 +459,8 @@ to C<use_socket>.
 
 =item C<< unlisten() >>
 
-Stop listening to the socket specified by use_socket or accept_on_fd.
+Stop listening on all sockets previously added via C<use_socket()> or
+C<accept_on_fd()>.  All listener file descriptors are closed.
 
 =item C<< pause_accept() >>
 
@@ -452,7 +477,9 @@ if resumed successfully, false if not paused or during shutdown.
 
 =item C<< accept_is_paused() >>
 
-Returns true if accepting is currently paused, false otherwise.
+Returns true if accepting is currently paused on all listeners, false
+otherwise.  With multiple listen sockets, all must be paused for this to
+return true.
 
 =item C<< request_handler(sub { my $req = shift; ... }) >>
 
@@ -461,10 +488,9 @@ Sets the global request handler.  Any previous handler is replaced.
 The handler callback is passed a L<Feersum::Connection> object.
 
 B<Subject to change>: if the request has an entity body then the handler will
-be called B<only> after receiving the body in its entirety.  The headers
-*must* specify a Content-Length of the body otherwise the request will be
-rejected.  The maximum size is hard coded to 2147483647 bytes (this may be
-considered a bug).
+be called B<only> after receiving the body in its entirety.  The body may use
+Content-Length or chunked Transfer-Encoding.  The maximum size defaults to
+67108864 bytes and can be changed via C<max_body_len()>.
 
 =item C<< psgi_request_handler(sub { my $env = shift; ... }) >>
 
@@ -474,24 +500,30 @@ Like request_handler, but assigns a PSGI handler instead.
 
 =item C<< read_timeout($duration) >>
 
-Get or set the global read timeout.
+Get or set the global read timeout.  Must be a positive non-zero value;
+passing 0 or a negative value will croak.  Changes take effect for new
+connections only; existing connections retain the timeout they were accepted
+with.
 
 Feersum will wait about this long to receive all headers of a request (within
 the tolerances provided by libev).  If an entity body is part of the request
 (e.g. POST or PUT) it will wait this long between successful C<read()> system
-calls.
+calls.  This timeout also serves as the keepalive idle timeout between
+requests on persistent connections; there is no separate setting for that.
 
 =item C<< header_timeout() >>
 
 =item C<< header_timeout($seconds) >>
 
 Get or set the header completion deadline timeout (Slowloris protection).
-Default is 0 (disabled).
+Default is 10 seconds.
 
 When enabled, connections must complete sending all HTTP headers within this
 many seconds from connection acceptance or receive a 408 Request Timeout
-response. This is a B<hard deadline> that does not reset when data arrives,
-unlike C<read_timeout> which resets on each successful read.
+response. For TLS connections where the handshake has not yet completed, the
+connection is silently closed (no HTTP response can be sent before the
+handshake finishes). This is a B<hard deadline> that does not reset when data
+arrives, unlike C<read_timeout> which resets on each successful read.
 
 This provides protection against Slowloris-style attacks where malicious
 clients send headers very slowly to exhaust server connection resources.
@@ -503,8 +535,9 @@ since the proxy handles slow clients.
 =item C<< graceful_shutdown(sub { .... }) >>
 
 Causes Feersum to initiate a graceful shutdown of all outstanding connections.
-No new connections will be accepted.  The reference to the socket provided
-in use_socket() is kept.
+No new connections will be accepted.  All listen socket file descriptors are
+closed; the Perl socket objects are not freed but the underlying fds are
+invalid after this call.
 
 The sub parameter is a completion callback.  It will be called when all
 connections have been flushed and closed.  This allows one to do something
@@ -531,15 +564,15 @@ The default implementation calls C<Carp::confess> which prints a full
 stack trace to STDERR. To install a custom handler:
 
     no strict 'refs';
-    *{'Feersum::DIED'} = sub { warn "Error: $_[0]" };
+    *{'Feersum::DIED'} = sub { warn "Error: $@" };
 
 Will get called for any errors that happen before the request handler callback
 is called, when the request handler callback throws an exception and
 potentially for other not-in-a-request-context errors.
 
-It will not get called for read timeouts that occur while waiting for a
-complete header (and also, until Feersum supports otherwise, time-outs while
-waiting for a request entity body).
+It will not get called for read timeouts or header deadline timeouts
+(Slowloris protection) that occur while waiting for a complete header, nor
+for timeouts while waiting for a request entity body.
 
 Note: Any exceptions thrown by the DIED handler itself are caught and will not
 propagate (the handler is called with G_EVAL). The server will still respond
@@ -551,15 +584,23 @@ Override Feersum's notion of what SERVER_NAME and SERVER_PORT should be.
 
 =item C<< set_keepalive($bool) >>
 
-Override Feersum's default keepalive behavior.
+Override Feersum's default keepalive behavior.  Changes take effect for new
+connections only.
 
 =item C<< set_reverse_proxy($bool) >>
 
-Enable or disable reverse proxy mode. When enabled, Feersum trusts
+Enable or disable reverse proxy mode.  Changes take effect for new
+connections only.  When enabled, Feersum trusts
 C<X-Forwarded-For> and C<X-Forwarded-Proto> headers from upstream proxies to
 determine the client's real IP address and request scheme.
 
-The L<Feersum::Connection> methods C<remote_addr()>, C<remote_port()>, and
+B<Security note:> Feersum uses the leftmost IP from C<X-Forwarded-For>,
+which assumes a single-hop reverse proxy that overwrites (not appends to)
+the header.  If your proxy appends to an existing C<X-Forwarded-For>,
+clients can spoof their IP by sending a forged header.  Ensure your
+reverse proxy strips or replaces C<X-Forwarded-For> rather than appending.
+
+The L<Feersum::Connection> methods C<remote_address()>, C<remote_port()>, and
 C<env()> will automatically use the forwarded values when this mode is active.
 
 =item C<< get_reverse_proxy() >>
@@ -653,8 +694,10 @@ connection load.
 
 =item C<< active_conns() >>
 
-Returns the current count of active connections being handled by Feersum.
-Useful for monitoring, load shedding decisions, or graceful shutdown logic.
+Returns the current count of active connection objects being handled by
+Feersum.  For HTTP/2, each concurrent stream counts as a separate unit in
+addition to the underlying TCP connection, so a single H2 connection with
+N streams contributes N+1 to this count.
 
 =item C<< total_requests() >>
 
@@ -665,17 +708,75 @@ Useful for monitoring and statistics. The counter is a 64-bit unsigned integer.
 
 =item C<< max_connections($limit) >>
 
-Get or set the maximum number of concurrent connections. Default is 0
-(unlimited).
+Get or set the maximum number of concurrent connections. Default is 10000.
 
-When the limit is reached, new connections are immediately closed after
-accept(). This provides protection against Slowloris-style DoS attacks that
-attempt to exhaust server resources by holding many connections open.
+When the limit is reached, Feersum first tries to close the oldest idle
+keep-alive connection to make room.  If no idle connections are available, the
+new connection is closed immediately after accept().  This provides protection
+against Slowloris-style DoS attacks that attempt to exhaust server resources by
+holding many connections open.
 
 Setting this to 0 disables the limit. In production, consider also running
 Feersum behind a reverse proxy (nginx, HAProxy) which can provide additional
 connection limiting and rate limiting.
 
+B<Note:> When HTTP/2 is in use, each H2 stream pseudo-connection counts
+against this limit in addition to the TCP connection itself.  See
+C<active_conns()>.
+
+=item C<< max_read_buf() >>
+
+=item C<< max_read_buf($bytes) >>
+
+Get or set the maximum read buffer size per connection (default 64 MB).
+This limits how large the read buffer can grow during header parsing and
+chunked body reception.  Requests that exceed the limit receive a 413
+response.  Set to 0 to reset to the compile-time default.
+
+=item C<< max_body_len() >>
+
+=item C<< max_body_len($bytes) >>
+
+Get or set the maximum request body size (default 64 MB).  This limits
+C<Content-Length> values and cumulative chunked body sizes.  Requests that
+exceed the limit receive a 413 response (HTTP/1.1) or RST_STREAM (HTTP/2).
+Set to 0 to reset to the compile-time default.
+
+=item C<< max_uri_len() >>
+
+=item C<< max_uri_len($bytes) >>
+
+Get or set the maximum request URI length (default 8192 bytes).  URIs that
+exceed the limit receive a 414 response.  Set to 0 to reset to the
+compile-time default.
+
+=item C<< write_timeout() >>
+
+=item C<< write_timeout($seconds) >>
+
+Get or set the write/response timeout.  Default is 0 (disabled).
+
+When enabled, connections that make no write progress within this many
+seconds are forcibly closed.  The timer resets on each successful write.
+This protects against slow consumers that stall the server by not reading
+response data.  For HTTP/2, the timeout operates per-stream: a stalled
+stream receives RST_STREAM rather than closing the entire connection.
+Disabled when the application takes over the socket via C<io()> or
+C<psgix.io>.
+
+=item C<< wbuf_low_water() >>
+
+=item C<< wbuf_low_water($bytes) >>
+
+Get or set the write buffer low-water-mark.  Default is 0 (callback fires
+only when the buffer is completely empty).
+
+When using streaming responses with C<poll_cb>, this setting controls when
+the write callback is invoked.  If set to a positive value, the callback
+fires when the buffered data drops to or below this threshold, allowing the
+application to keep the write pipe full for better throughput.  Works across
+all transports (plain, TLS, and HTTP/2).
+
 =item C<< set_tls(cert_file => $path, key_file => $path, [listener => $idx]) >>
 
 Enable TLS 1.3 on a listener. Requires Feersum to be compiled with TLS
@@ -689,16 +790,17 @@ The optional C<listener> parameter specifies which listener to configure
 Defaults to the last-added listener.
 
 Call this after C<use_socket()> or C<accept_on_fd()> to apply TLS to that
-listener. Different listeners can have different TLS configurations, or some
-can be plain HTTP while others use TLS.
+listener.  Croaks if no listeners have been configured yet.  Different
+listeners can have different TLS configurations, or some can be plain HTTP
+while others use TLS.
 
     my $ngn = Feersum->endjinn;
     $ngn->use_socket($tls_socket);
     $ngn->set_tls(cert_file => 'server.crt', key_file => 'server.key');
 
 When TLS is enabled and L<Alien::nghttp2> was available at build time,
-HTTP/2 can be enabled by passing C<< h2 => 1 >> (HTTP/2 is off by
-default):
+HTTP/2 can be enabled by passing C<< h2 => 1 >>.  Without this flag, only
+C<http/1.1> is offered during ALPN negotiation:
 
     $ngn->set_tls(cert_file => 'server.crt', key_file => 'server.key',
                   h2 => 1);
@@ -715,6 +817,38 @@ Returns true if Feersum was compiled with HTTP/2 support (nghttp2).
 
 =back
 
+=head1 OBSERVABILITY
+
+Feersum includes static USDT (Userland Statically Defined Tracing) probes
+for high-performance observability via DTrace or eBPF (bpftrace).
+
+Probes provided by the C<feersum> provider:
+
+=over 4
+
+=item C<conn_new(fd, remote_addr, remote_port)>
+
+Fired when a new TCP connection is accepted.
+
+=item C<conn_free(fd)>
+
+Fired when a connection is closed and its resources are freed.
+
+=item C<req_new(fd, method, uri)>
+
+Fired when a complete set of HTTP headers has been parsed and a new request
+is starting.
+
+=item C<req_body(fd, length)>
+
+Fired when a chunk of the request entity body is received.
+
+=item C<resp_start(fd, status_code)>
+
+Fired when the response begins (headers are being sent).
+
+=back
+
 =cut
 
 =head1 GRITTY DETAILS
@@ -736,7 +870,7 @@ If a request exceeds this limit, a 400 response is given and the app handler doe
 
 Defaults to 128.  Controls how long the name of each header can be.
 
-If a request exceeds this limit, a 400 response is given and the app handler does not run.
+If a request exceeds this limit, a 431 response is given and the app handler does not run.
 
 =item MAX_URI_LEN
 
@@ -748,12 +882,11 @@ does not run.
 
 =item MAX_BODY_LEN
 
-Defaults to ~2GB.  Controls how large the body of a POST/PUT/etc. can be when
-that request has a C<Content-Length> header.
+Compile-time default for C<max_body_len()> (64 MB).  Controls how large the
+body of a POST/PUT/etc. can be.  Use C<max_body_len($bytes)> to override at
+runtime.
 
-If a request exceeds this limit, a 413 response is given and the app handler does not run.
-
-See also BUGS
+See also L</BUGS>.
 
 =item READ_BUFSZ
 
@@ -770,24 +903,10 @@ If another read is needed and the buffer is under READ_BUFSZ bytes
 then the buffer gets an additional READ_GROW_FACTOR * READ_BUFSZ bytes.
 The trade-off with the grow factor is memory usage vs. system calls.
 
-=item AUTOCORK_WRITES
-
-Controls how response data is written to sockets.  If enabled, the event loop
-is used to wait until the socket is writable; if disabled (the default), a
-write is performed immediately.  In either case, non-blocking writes are used.
-Using the event loop is "nicer" but perhaps introduces latency, hence this
-option.  Set to 1 in Feersum.xs to enable event-loop-based writes.
-
-=item KEEPALIVE_CONNECTION
-
-Vestigial compile-time constant (default 0).  Keepalive is now controlled at
-runtime via C<< $server->set_keepalive(1) >>.  When enabled, "Connection:
-keep-alive" for HTTP/1.0 and "Connection: close" for HTTP/1.1 are
-acknowledged.
-
 =item READ_TIMEOUT
 
-Controls read timeout. Default is 5.0 sec. It is also an keepalive timeout.
+Controls read timeout. Default is 5.0 sec. Also used as the keepalive idle
+timeout (there is no separate keepalive timeout setting).
 
 =item FEERSUM_IOMATRIX_SIZE
 
@@ -841,7 +960,9 @@ handshake.
 
 B<Request methods> -- all standard methods (GET, POST, PUT, DELETE, etc.)
 are supported.  Request bodies are fully buffered before the handler is
-called, same as HTTP/1.x.
+called, same as HTTP/1.x.  B<Note:> unlike HTTP/1.1 where Feersum rejects
+non-standard methods (TRACE, PROPFIND, etc.) with 405, HTTP/2 passes all
+methods through to the request handler.
 
 =item *
 
@@ -856,8 +977,8 @@ parallel on a single connection, up to C<FEER_H2_MAX_CONCURRENT_STREAMS>
 
 =item *
 
-B<Not supported> -- server push, trailers, streaming (incremental) request
-bodies, and C<sendfile>.  For HTTP/2 responses, use C<write()> instead of
+B<Not supported> -- server push, server-sent trailers, streaming (incremental)
+request bodies, and C<sendfile>.  For HTTP/2 responses, use C<write()> instead of
 C<sendfile>.
 
 =item *
@@ -869,15 +990,25 @@ C<SERVER_PROTOCOL> is C<HTTP/2>.
 
 B<Extended CONNECT / WebSocket tunnels (RFC 8441)> -- Feersum advertises
 C<SETTINGS_ENABLE_CONNECT_PROTOCOL=1> so HTTP/2 clients can open WebSocket
-tunnels via Extended CONNECT.  The PSGI handler sees:
+tunnels via Extended CONNECT.  Feersum translates the H2 Extended CONNECT
+into H1-equivalent PSGI env variables (matching HAProxy/nghttpx behaviour),
+so existing PSGI WebSocket middleware works transparently:
 
-    REQUEST_METHOD       => 'CONNECT'
+    REQUEST_METHOD       => 'GET'             # translated from CONNECT
     HTTP_UPGRADE         => 'websocket'       # synthesised from :protocol
+    HTTP_CONNECTION      => 'Upgrade'         # synthesised
     psgix.h2.protocol    => 'websocket'       # raw :protocol value
     psgix.h2.extended_connect => 1
 
-The handler accepts or rejects the tunnel by calling C<start_streaming>
-with a C<200> status (accept) or a C<4xx> status (reject).  Once accepted,
+The native C<< $req->method() >> likewise returns C<GET> for these streams.
+
+The handler code is identical to HTTP/1.1 upgrades: write an C<HTTP/1.1 101>
+response line followed by C<Upgrade:> / C<Connection:> headers via C<psgix.io>
+(or C<< $req->io() >>).  Under H2, Feersum automatically sends 200 HEADERS to
+accept the tunnel and silently swallows the HTTP/1.1 101 response written by
+the app, relaying only the subsequent data as H2 DATA frames.  This means the
+same PSGI handler works for both H1 and H2 without any protocol branching.
+
 C<< psgix.io >> (or C<< $req->io() >>) returns a bidirectional handle backed
 by a Unix socketpair; Feersum bridges bytes between that handle and the
 HTTP/2 DATA frames in both directions.
@@ -888,10 +1019,9 @@ HTTP/2 DATA frames in both directions.
 
 Please report bugs using http://github.com/stash/Feersum/issues/
 
-Currently there's no way to limit the request entity length of a B<streaming>
-POST/PUT/etc.  This could lead to a DoS attack on a Feersum server.  Suggested
-remedy is to only run Feersum behind some other web server and to use that to
-limit the entity size.
+Request bodies are capped at C<MAX_BODY_LEN> (64 MB by default).  For
+untrusted clients it is still recommended to run Feersum behind a reverse
+proxy that enforces tighter entity-size limits.
 
 Although not explicitly a bug, the following may cause undesirable behavior.
 Feersum will have set SIGPIPE to be ignored by the time your handler gets
