@@ -8,25 +8,6 @@ sub new {
     croak "Cannot instantiate Feersum::Connection directly";
 }
 
-sub read_handle {
-    croak "read_handle is deprecated; use psgi.input instead";
-}
-
-sub write_handle {
-    croak "write_handle is deprecated; ".
-        "use return value from start_streaming instead";
-}
-
-sub start_response {
-    croak "start_response is deprecated; ".
-        "use start_streaming() or send_response() instead";
-}
-
-sub initiate_streaming {
-    croak "initiate_streaming is deprecated; ".
-        "use start_streaming() and its return value instead";
-}
-
 sub _initiate_streaming_psgi {
     my ($self, $streamer) = @_;
     return $streamer->(sub { $self->_continue_streaming_psgi(@_) });
@@ -168,7 +149,9 @@ bidirectional and suitable for WebSocket or other tunnel protocols.
 
 B<WARNING>: Once you call this method, Feersum relinquishes control of the
 socket. You are responsible for all I/O and must not use other Feersum
-response methods on this connection.
+response methods on this connection.  B<Do not> call C<io()> on regular
+(non-tunnel) HTTP/2 streams — it would expose the shared TCP socket
+underlying all multiplexed streams on that connection.
 
 =item C<< $req->return_from_io($io) >>
 
@@ -211,7 +194,9 @@ body content length (psgi CONTENT_LENGTH)
 
 =item C<< my $input = $req->input >>
 
-input body handler (psgi.input), it is advised to close it after read is done
+Input body handler (psgi.input).  Returns C<undef> for requests with no body
+(e.g., GET, HEAD) — check the return value before calling C<read()>.  It is
+advised to close it after read is done.
 
 =item C<< my $headers = $req->headers([normalization_style]) >>
 
@@ -233,12 +218,14 @@ simple lookup for header value, name should be in lowercase, eg. 'content-type'
 
 =item C<< my $addr = $req->remote_address >>
 
-Remote address of the connection (psgi REMOTE_ADDR). This always returns
-the actual socket peer address.
+Remote address of the connection (psgi REMOTE_ADDR).  When PROXY protocol is
+active, returns the client address from the PROXY header; otherwise returns
+the socket peer address.
 
 =item C<< my $port = $req->remote_port >>
 
-Remote port of the connection (psgi REMOTE_PORT).
+Remote port of the connection (psgi REMOTE_PORT).  When PROXY protocol is
+active, returns the client port from the PROXY header.
 
 =item C<< my $addr = $req->client_address >>
 
@@ -260,11 +247,17 @@ numbers (as integers), values are raw TLV data bytes. Only populated
 when C<proxy_protocol> is enabled and the client sends a v2 header
 with TLV extensions (e.g. PP2_TYPE_SSL, PP2_TYPE_AUTHORITY).
 
+=item C<< my $trailers = $req->trailers >>
+
+Returns an array reference of request trailers in form of [ name => value, ... ],
+or C<undef> if no trailers were received. Only supported for HTTP/2
+requests currently.
+
 =back
 
 =begin comment
 
-=head2 Private and or Deprecated Methods
+=head2 Private Methods
 
 =over 4
 
@@ -272,20 +265,6 @@ with TLV extensions (e.g. PP2_TYPE_SSL, PP2_TYPE_AUTHORITY).
 
 No-op. Feersum will create these objects internally.
 
-=item C<< $req->read_handle >>
-
-use psgi.input instead
-
-=item C<< $req->write_handle >>
-
-=item C<< $req->start_response(...) >>
-
-use start_streaming() or send_response() instead
-
-=item C<< $req->initiate_streaming(...) >>
-
-use start_streaming() and its return value instead
-
 =back
 
 =end comment
