@@ -92,13 +92,15 @@ hash.
 =item C<< $r->seek(...) >>
 
 Seeking is partially supported.  Feersum discards skipped-over bytes to
-conserve memory.
+conserve memory.  B<Note:> SEEK_SET is treated the same as SEEK_CUR (always
+relative to the current position), since the underlying buffer is consumed
+as it is read and absolute positioning is not supported.
 
     $r->seek(0,SEEK_CUR);  # returns 1
     $r->seek(-1,SEEK_CUR); # returns 0
     $r->seek(-1,SEEK_SET); # returns 0
-    $r->seek(2,SEEK_CUR); # returns 1, discards skipped bytes
-    $r->seek(42,SEEK_SET); # returns 1 if room, discards skipped bytes
+    $r->seek(2,SEEK_CUR);  # returns 1, discards 2 bytes
+    $r->seek(42,SEEK_SET); # same as SEEK_CUR: discards 42 bytes
     $r->seek(-8,SEEK_END); # returns 1 if room, discards skipped bytes
 
 =item C<< $r->close() >>
@@ -155,6 +157,12 @@ A reference to the writer is passed in as the first and only argument to the
 sub.  It's recommended that you use C<$_[0]> rather than closing-over on C<$w>
 to prevent a circular reference.
 
+=item C<< $w->return_from_psgix_io($io) >>
+
+Returns control of the socket back to Feersum after C<psgix.io> was used.
+This is the PSGI-handle equivalent of C<< $req->return_from_io($io) >> on
+the connection object.  See L<Feersum::Connection/return_from_io> for details.
+
 =item C<< $w->sendfile($fh [, $offset, $length]) >>
 
 Send file contents using zero-copy sendfile(2) system call. Linux only.
