@@ -195,6 +195,7 @@ sub _prepare {
         $self->{$_} = delete $opts->{$_} for grep defined($opts->{$_}),
             qw/pre_fork keepalive read_timeout header_timeout max_connection_reqs reuseport epoll_exclusive
                read_priority write_priority accept_priority max_accept_per_loop max_connections
+               max_read_buf max_body_len max_uri_len wbuf_low_water
                reverse_proxy proxy_protocol h2 tls tls_cert_file tls_key_file/;
         # Warn about unknown options (likely typos)
         for my $unknown (keys %$opts) {
@@ -265,6 +266,10 @@ sub _prepare {
             unless $val =~ /^\d+$/;
     }
     $f->max_connections($_) for grep defined, delete $self->{max_connections};
+    $f->max_read_buf($_) for grep defined, delete $self->{max_read_buf};
+    $f->max_body_len($_) for grep defined, delete $self->{max_body_len};
+    $f->max_uri_len($_) for grep defined, delete $self->{max_uri_len};
+    $f->wbuf_low_water($_) for grep defined, delete $self->{wbuf_low_water};
 
     # Build tls hash from flat options (for Plack -o tls_cert_file=... -o tls_key_file=...)
     if (!$self->{tls}) {
@@ -286,6 +291,10 @@ sub _prepare {
         croak "tls must be a hash reference" unless ref $tls eq 'HASH';
         croak "tls requires cert_file" unless $tls->{cert_file};
         croak "tls requires key_file" unless $tls->{key_file};
+        -f $tls->{cert_file} && -r _
+            or croak "tls cert_file '$tls->{cert_file}': not found or not readable";
+        -f $tls->{key_file} && -r _
+            or croak "tls key_file '$tls->{key_file}': not found or not readable";
 
         # H2 is off by default; only enable if h2 => 1 was passed
         if (delete $self->{h2}) {
@@ -357,6 +366,7 @@ sub _fork_another {
         $self->{quiet} or warn "Feersum [$$]: starting\n";
         delete $self->{_kids};
         delete $self->{pre_fork};
+        $self->{_n_kids} = 0;
 
         # With SO_REUSEPORT, each child creates its own sockets
         # This eliminates accept() contention for better scaling
@@ -367,11 +377,17 @@ sub _fork_another {
                     or do { warn "close parent socket in child: $!"; POSIX::_exit(1); };
             }
             my @new_socks;
-            for my $listen (@{$self->{_listen_addrs}}) {
-                my $sock = $self->_create_socket($listen, 1);
-                push @new_socks, $sock;
-                $self->{endjinn}->use_socket($sock);
-            }
+            eval {
+                for my $listen (@{$self->{_listen_addrs}}) {
+                    my $sock = $self->_create_socket($listen, 1);
+                    push @new_socks, $sock;
+                    $self->{endjinn}->use_socket($sock);
+                }
+                1;
+            } or do {
+                warn "Feersum [$$]: child socket creation failed: $@";
+                POSIX::_exit(1);
+            };
             $self->{sock} = $new_socks[0];
             $self->{_socks} = \@new_socks;
         }
@@ -605,6 +621,21 @@ Set maximum concurrent connections (default: 0, unlimited). When the limit is
 reached, new connections are immediately closed. Provides protection against
 Slowloris-style DoS attacks.
 
+=item max_read_buf
+
+Set max read buffer size per connection (default: 64 MB).  This limits how
+large the read buffer can grow during header parsing and chunked body
+reception.
+
+=item max_body_len
+
+Set max request body size (default: 64 MB).  This limits Content-Length
+values and cumulative chunked body sizes.
+
+=item max_uri_len
+
+Set max request URI length (default: 8192).
+
 =item read_priority
 
 =item write_priority
